@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License version 2,
@@ -34,6 +34,7 @@
 #include <KConfigDialog>
 #include <KProcess>
 #include <plasma/animator.h>
+#include <plasma/windoweffects.h>
 
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusReply>
@@ -80,7 +81,6 @@ LancelotWindow::LancelotWindow()
     : m_root(NULL), m_corona(NULL),
     m_hovered(false), m_showingFull(true), m_sectionsSignalMapper(NULL),
     m_config("lancelotrc"), m_mainConfig(&m_config, "Main"),
-    instance(NULL),
     m_configWidget(NULL),
     m_resizeDirection(None),
     m_mainSize(mainWidthDefault, windowHeightDefault),
@@ -105,9 +105,6 @@ LancelotWindow::LancelotWindow()
     setMinimumSize(400, 300);
     setScene(m_corona);
 
-    instance = new Lancelot::Instance();
-    instance->setHasApplication(true);
-
     m_root = new Lancelot::ResizeBordersPanel();
     installEventFilter(this);
 
@@ -124,10 +121,11 @@ LancelotWindow::LancelotWindow()
     editSearch->setParentItem(m_root);
     editSearch->nativeWidget()->setClearButtonShown(true);
     editSearch->nativeWidget()->setClickMessage(i18n("Search"));
+    editSearch->nativeWidget()->setContextMenuPolicy(Qt::NoContextMenu);
     editSearch->show();
     layoutSearch->addItem(editSearch,
         Lancelot::NodeLayout::NodeCoordinate(0.0, 0.5, 0, 0),
-        Lancelot::NodeLayout::NodeCoordinate(1.0, 0.5, 0, INFINITY)
+        Lancelot::NodeLayout::NodeCoordinate(1.0, 0.5, 0, QREAL_INFINITY)
     );
     editSearch->nativeWidget()->installEventFilter(this);
     editSearch->setFocusPolicy(Qt::WheelFocus);
@@ -139,8 +137,6 @@ LancelotWindow::LancelotWindow()
     passagewayApplications->setAtlasIcon(KIcon("applications-other"));
     /* End TODO */
 
-    // instance->activateAll();
-
     m_sectionsSignalMapper = new QSignalMapper(this);
     connect (m_sectionsSignalMapper,
         SIGNAL(mapped(const QString &)),
@@ -166,18 +162,22 @@ LancelotWindow::LancelotWindow()
         this, SLOT(search(const QString &))
     );
 
-    instance->activateAll();
     loadConfig();
     setupActions();
 
     connect(QApplication::instance(), SIGNAL(focusChanged(QWidget *, QWidget *)),
             this, SLOT(focusChanged(QWidget *, QWidget *)));
+
+    setStyleSheet("LancelotWindow { background: black }");
 }
 
 void LancelotWindow::drawBackground(QPainter * painter, const QRectF & rect)
 {
     painter->setCompositionMode(QPainter::CompositionMode_Clear);
-    painter->fillRect(QRectF(rect.x()-2,rect.y()-2,rect.width()+2,rect.height()+2).toRect(), Qt::transparent);
+    painter->fillRect(
+        QRectF(rect.x() - 2, rect.y() - 2,
+               rect.width() + 2, rect.height() + 2).toRect(),
+               Qt::transparent);
     painter->setCompositionMode(QPainter::CompositionMode_Source);
 }
 
@@ -197,7 +197,6 @@ void LancelotWindow::focusChanged(QWidget * old, QWidget * now)
 LancelotWindow::~LancelotWindow()
 {
     delete m_configWidget;
-    delete instance;
 }
 
 void LancelotWindow::lancelotShow(int x, int y)
@@ -276,6 +275,14 @@ void LancelotWindow::showWindow(int x, int y, bool centered)
             y = y - height();
             flip &= ~Plasma::VerticalFlip;
         }
+
+        if (m_showingFull) {
+            if (flip & Plasma::HorizontalFlip) {
+                x += sectionsWidth / 2;
+            } else {
+                x -= sectionsWidth / 2;
+            }
+        }
     } else {
         flip = Plasma::NoFlip;
 
@@ -289,30 +296,39 @@ void LancelotWindow::showWindow(int x, int y, bool centered)
     layoutSections->setFlip(flip);
 
     if (m_configUi.activationMethod() == LancelotConfig::NoClick) {
-        instance->group("SystemButtons")->setProperty("ExtenderPosition", QVariant(
+        Lancelot::Global::self()->group("SystemButtons")->setProperty("extenderPosition", QVariant(
                 (flip & Plasma::VerticalFlip)?(Lancelot::TopExtender):(Lancelot::BottomExtender)
         ));
-        instance->group("SystemButtons")
-            ->setProperty("ActivationMethod", Lancelot::ExtenderActivate);
+        Lancelot::Global::self()->group("SystemButtons")
+            ->setProperty("activationMethod", Lancelot::ExtenderActivate);
     } else {
-        instance->group("SystemButtons")
-            ->setProperty("ExtenderPosition", QVariant(Lancelot::NoExtender));
-        instance->group("SystemButtons")
-            ->setProperty("ActivationMethod", Lancelot::ClickActivate);
+        Lancelot::Global::self()->group("SystemButtons")
+            ->setProperty("extenderPosition", QVariant(Lancelot::NoExtender));
+        Lancelot::Global::self()->group("SystemButtons")
+            ->setProperty("activationMethod", Lancelot::ClickActivate);
     }
-    instance->group("LancelotContext")->setProperty("ExtenderPosition",
-            instance->group("SystemButtons")->property("ExtenderPosition"));
-    instance->group("LancelotContext")->setProperty("ActivationMethod",
-            instance->group("SystemButtons")->property("ActivationMethod"));
-    instance->group("SystemButtons")->notifyUpdated();
-    instance->group("LancelotContext")->notifyUpdated();
+    Lancelot::Global::self()->group("LancelotContext")->setProperty("extenderPosition",
+            Lancelot::Global::self()->group("SystemButtons")->property("extenderPosition"));
+    Lancelot::Global::self()->group("LancelotContext")->setProperty("activationMethod",
+            Lancelot::Global::self()->group("SystemButtons")->property("activationMethod"));
+    Lancelot::Global::self()->group("SystemButtons")->notifyUpdated();
+    Lancelot::Global::self()->group("LancelotContext")->notifyUpdated();
 
     if (m_showingFull) {
         sectionActivated("applications");
     }
 
     move(x, y);
+    if (KWindowSystem::compositingActive()) {
+        if (flip & Plasma::VerticalFlip) {
+            Plasma::WindowEffects::slideWindow(this, Plasma::TopEdge);
+
+        } else {
+            Plasma::WindowEffects::slideWindow(this, Plasma::BottomEdge);
+        }
+    }
     show();
+
     KWindowSystem::setState( winId(), NET::SkipTaskbar | NET::SkipPager | NET::KeepAbove );
     KWindowSystem::forceActiveWindow(winId());
 
@@ -420,14 +436,14 @@ void LancelotWindow::systemButtonClicked()
         static_cast < Lancelot::ExtenderButton * > (sender());
 
     Lancelot::ActionTreeModel * model =
-        Models::SystemActions::instance()->action(systemButtonActions[button]);
+        Models::SystemActions::self()->action(systemButtonActions[button]);
 
     if (!model) return;
 
     if (!menuSystemButton) {
         menuSystemButton = new Lancelot::PopupList();
         menuSystemButton->resize(200, 200);
-        Models::ApplicationConnector * ac = Models::ApplicationConnector::instance();
+        Models::ApplicationConnector * ac = Models::ApplicationConnector::self();
         connect(
                 ac, SIGNAL(doHide(bool)),
                 menuSystemButton, SLOT(close())
@@ -464,7 +480,26 @@ void LancelotWindow::setupModels()
     // m_models["Contacts"]          = new Models::ContactsKopete();
     // m_models["Messages"]          = new Models::MessagesKmail();
 
-    m_models["Runner"]            = new Models::Runner();
+    QStringList allowedRunners = m_mainConfig.readEntry("allowedRunners", QStringList());
+    if (allowedRunners.isEmpty()) {
+        allowedRunners
+            << "places"
+            << "windows"
+            << "kill"
+            << "solid"
+            << "services"
+            << "PowerDevil"
+            << "browserhistory"
+            << "shell"
+            << "locations"
+            << "konsolesessions"
+            << "recentdocuments"
+            << "calculator"
+            << "bookmarks"
+            << "unitconverter";
+    }
+
+    m_models["Runner"]            = new Models::Runner(allowedRunners);
 
     // Groups:
 
@@ -501,7 +536,7 @@ void LancelotWindow::setupModels()
         MergedAddModel(m_modelGroups["ContactsLeft"], "Messages", m_models["Messages"]);
     } else if (plugins != "disabled") {
         Lancelot::ActionListModel * model;
-        foreach (QString plugin, plugins.split(',')) {
+        foreach (const QString &plugin, plugins.split(',')) {
             model = new Lancelot::PlasmaServiceListModel(plugin);
             MergedAddModel(m_modelGroups["ContactsLeft"], model->selfTitle(), model);
         }
@@ -514,7 +549,7 @@ void LancelotWindow::setupModels()
         MergedAddModel(m_modelGroups["ContactsRight"], "Contacts", m_models["Contacts"]);
     } else if (plugins != "disabled") {
         Lancelot::ActionListModel * model;
-        foreach (QString plugin, plugins.split(',')) {
+        foreach (const QString &plugin, plugins.split(',')) {
             model = new Lancelot::PlasmaServiceListModel(plugin);
             MergedAddModel(m_modelGroups["ContactsRight"], model->selfTitle(), model);
         }
@@ -646,6 +681,7 @@ bool LancelotWindow::eventFilter(QObject * object, QEvent * event)
                 }
             case Qt::Key_Up:
             case Qt::Key_Down:
+            case Qt::Key_Menu:
                 sendKeyEvent(keyEvent);
                 break;
             case Qt::Key_Left:
@@ -654,6 +690,12 @@ bool LancelotWindow::eventFilter(QObject * object, QEvent * event)
             case Qt::Key_Right:
                 m_focusIndex++;
                 break;
+            case Qt::Key_PageUp:
+                nextSection(-1);
+                return true;
+            case Qt::Key_PageDown:
+                nextSection(1);
+                return true;
             default:
                 pass = true;
         }
@@ -679,7 +721,6 @@ bool LancelotWindow::eventFilter(QObject * object, QEvent * event)
             sendKeyEvent(keyEvent);
         }
 
-
         editSearch->nativeWidget()->setFocus();
         editSearch->setFocus();
     }
@@ -731,6 +772,25 @@ void LancelotWindow::configurationChanged()
     loadConfig();
 }
 
+void LancelotWindow::nextSection(int increment)
+{
+    qDebug() << "LancelotWindow::nextSection " << increment;
+    if (! (layoutMain->flip() & Plasma::VerticalFlip)) {
+        increment = - increment;
+    }
+
+    int currentIndex = sectionIDs().indexOf(m_activeSection);
+    currentIndex += increment;
+
+    if (currentIndex >= sectionIDs().size()) {
+        currentIndex = 0;
+    } else if (currentIndex < 0) {
+        currentIndex = sectionIDs().size() - 1;
+    }
+
+    sectionActivated(sectionIDs()[currentIndex]);
+}
+
 void LancelotWindow::loadConfig()
 {
     // Non configurable options
@@ -766,60 +826,62 @@ void LancelotWindow::loadConfig()
     }
 
     if (systemNoClick) {
-        instance->group("SystemButtons")->setProperty("ExtenderPosition",
+        Lancelot::Global::self()->group("SystemButtons")->setProperty("extenderPosition",
                 (layoutMain->flip() & Plasma::VerticalFlip)
                     ? (Lancelot::TopExtender) : (Lancelot::BottomExtender)
         );
-        instance->group("SystemButtons")
-            ->setProperty("ActivationMethod", Lancelot::ExtenderActivate);
+        Lancelot::Global::self()->group("SystemButtons")
+            ->setProperty("activationMethod", Lancelot::ExtenderActivate);
     } else {
-        instance->group("SystemButtons")
-            ->setProperty("ExtenderPosition", QVariant(Lancelot::NoExtender));
-        instance->group("SystemButtons")
-            ->setProperty("ActivationMethod", Lancelot::ClickActivate);
+        Lancelot::Global::self()->group("SystemButtons")
+            ->setProperty("extenderPosition", QVariant(Lancelot::NoExtender));
+        Lancelot::Global::self()->group("SystemButtons")
+            ->setProperty("activationMethod", Lancelot::ClickActivate);
     }
 
-    instance->group("LancelotContext")->setProperty("ExtenderPosition",
-            instance->group("SystemButtons")->property("ExtenderPosition"));
-    instance->group("LancelotContext")->setProperty("ActivationMethod",
-            instance->group("SystemButtons")->property("ActivationMethod"));
-    instance->group("SystemButtons")->notifyUpdated();
-    instance->group("LancelotContext")->notifyUpdated();
+    Lancelot::Global::self()->group("LancelotContext")->setProperty("extenderPosition",
+            Lancelot::Global::self()->group("SystemButtons")->property("extenderPosition"));
+    Lancelot::Global::self()->group("LancelotContext")->setProperty("activationMethod",
+            Lancelot::Global::self()->group("SystemButtons")->property("activationMethod"));
+    Lancelot::Global::self()->group("SystemButtons")->notifyUpdated();
+    Lancelot::Global::self()->group("LancelotContext")->notifyUpdated();
 
     if (sectionNoClick) {
-        instance->group("SectionButtons")
-            ->setProperty("ActivationMethod", Lancelot::HoverActivate);
+        Lancelot::Global::self()->group("SectionButtons")
+            ->setProperty("activationMethod", Lancelot::HoverActivate);
     } else {
-        instance->group("SectionButtons")
-            ->setProperty("ActivationMethod", Lancelot::ClickActivate);
+        Lancelot::Global::self()->group("SectionButtons")
+            ->setProperty("activationMethod", Lancelot::ClickActivate);
     }
-    instance->group("SectionButtons")->notifyUpdated();
+    Lancelot::Global::self()->group("SectionButtons")->notifyUpdated();
 
     if (listsNoClick) {
-        instance->group("ActionListView-Left")
-            ->setProperty("ExtenderPosition", Lancelot::LeftExtender);
-        instance->group("ActionListView-Right")
-            ->setProperty("ExtenderPosition", Lancelot::RightExtender);
-        instance->group("PassagewayView")
-            ->setProperty("ActivationMethod", Lancelot::ExtenderActivate);
-        instance->group("PopupList")
-            ->setProperty("ExtenderPosition", Lancelot::RightExtender);
+        Lancelot::Global::self()->group("ActionListView-Left")
+            ->setProperty("extenderPosition", Lancelot::LeftExtender);
+        Lancelot::Global::self()->group("ActionListView-Right")
+            ->setProperty("extenderPosition", Lancelot::RightExtender);
+        Lancelot::Global::self()->group("PassagewayView")
+            ->setProperty("activationMethod", Lancelot::ExtenderActivate);
+        Lancelot::Global::self()->group("PopupList")
+            ->setProperty("extenderPosition", Lancelot::RightExtender);
     } else {
-        instance->group("ActionListView-Left")
-            ->setProperty("ExtenderPosition", Lancelot::NoExtender);
-        instance->group("ActionListView-Right")
-            ->setProperty("ExtenderPosition", Lancelot::NoExtender);
-        instance->group("PassagewayView")
-            ->setProperty("ActivationMethod", Lancelot::ClickActivate);
-        instance->group("PopupList")
-            ->setProperty("ExtenderPosition", Lancelot::NoExtender);
+        Lancelot::Global::self()->group("ActionListView-Left")
+            ->setProperty("extenderPosition", Lancelot::NoExtender);
+        Lancelot::Global::self()->group("ActionListView-Right")
+            ->setProperty("extenderPosition", Lancelot::NoExtender);
+        Lancelot::Global::self()->group("PassagewayView")
+            ->setProperty("activationMethod", Lancelot::ClickActivate);
+        Lancelot::Global::self()->group("PopupList")
+            ->setProperty("extenderPosition", Lancelot::NoExtender);
     }
-    instance->group("ActionListView-Left")->notifyUpdated();
-    instance->group("ActionListView-Right")->notifyUpdated();
-    instance->group("PassagewayView")->notifyUpdated();
+    Lancelot::Global::self()->group("ActionListView-Left")->notifyUpdated();
+    Lancelot::Global::self()->group("ActionListView-Right")->notifyUpdated();
+    Lancelot::Global::self()->group("PassagewayView")->notifyUpdated();
 
     // PassagewayView settings
-    if (m_configUi.appbrowserColumnLimitted()) {
+    if (m_configUi.appbrowserPopupSubmenus()) {
+        passagewayApplications->setColumnLimit(-1);
+    } else if (m_configUi.appbrowserColumnLimitted()) {
         passagewayApplications->setColumnLimit(2);
     } else {
         passagewayApplications->setColumnLimit(22); // TODO: Temp
@@ -834,17 +896,17 @@ void LancelotWindow::loadConfig()
     while (i.hasNext()) {
         i.next();
 
-        i.key()->setTitle(Models::SystemActions::instance()->actionTitle(
+        i.key()->setTitle(Models::SystemActions::self()->actionTitle(
                     i.value()));
-        i.key()->setIcon(Models::SystemActions::instance()->actionIcon(
+        i.key()->setIcon(Models::SystemActions::self()->actionIcon(
                     i.value()));
     }
 
     // Logging
-    Logger::instance()->setEnabled(m_configUi.enableUsageStatistics());
+    Logger::self()->setEnabled(m_configUi.enableUsageStatistics());
 
     // Keep open
-    Models::ApplicationConnector::instance()->setAutohideEnabled(!m_configUi.checkKeepOpen->isChecked());
+    Models::ApplicationConnector::self()->setAutohideEnabled(!m_configUi.checkKeepOpen->isChecked());
 }
 
 void LancelotWindow::lancelotContext()
