@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License version 2,
@@ -17,10 +17,11 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#ifndef LANCELOTAPP_MODELS_RUNNER_H_
-#define LANCELOTAPP_MODELS_RUNNER_H_
+#ifndef LANCELOTAPP_MODELS_RUNNER_H
+#define LANCELOTAPP_MODELS_RUNNER_H
 
 #include "BaseModel.h"
+#include <QBasicTimer>
 #include <plasma/runnermanager.h>
 #include <plasma/querymatch.h>
 
@@ -29,14 +30,22 @@ namespace Models {
 class Runner : public BaseModel {
     Q_OBJECT
 public:
-    Runner(QString search = QString());
+    explicit Runner(bool limitRunners = false, QString search = QString());
+    explicit Runner(QStringList allowedRunners, QString search = QString());
     virtual ~Runner();
 
-    QString searchString();
+    QString searchString() const;
 
-    L_Override virtual bool hasContextActions(int index) const;
-    L_Override virtual void setContextActions(int index, QMenu * menu);
-    L_Override virtual void contextActivate(int index, QAction * context);
+    QString runnerName() const;
+    void setRunnerName(const QString & name);
+
+    L_Override bool hasContextActions(int index) const;
+    L_Override void setContextActions(int index, Lancelot::PopupMenu * menu);
+    L_Override void contextActivate(int index, QAction * context);
+
+    L_Override QMimeData * mimeData(int index) const;
+    L_Override void setDropActions(int index,
+            Qt::DropActions & actions, Qt::DropAction & defaultAction);
 
 public Q_SLOTS:
     void setSearchString(const QString & search);
@@ -45,13 +54,16 @@ public Q_SLOTS:
 protected:
     void activate(int index);
     void load();
+    void timerEvent(QTimerEvent * event);
 
 private:
     QString m_searchString;
+    QString m_runnerName;
+    QBasicTimer m_timer;
     Plasma::RunnerManager * m_runnerManager;
     bool valid : 1;
 };
 
 } // namespace Models
 
-#endif /* LANCELOTAPP_MODELS_DEVICES_H_ */
+#endif /* LANCELOTAPP_MODELS_DEVICES_H */
